/**
 * Very simple stand-alone program to compute the mean of two integers.
 *
 */
public final class Mean {

    /** Disabled default constructor. */
    private Mean() {
        throw new Error("Mean cannot be instantiated");
    }

    /**
     * Main program entry point.
     *
     * Computes the mean of the two integers passed as command line arguments
     * and prints it to the standard output stream.  If the arguments are
     * invalid, an error message is printed to the standard error output stream
     * and the program is aborted with exit status 1.
     *
     * @param args
     *         command line arguments (exactly two integers)
     *
     */
    public static void main(final String[] args) {
        boolean fail = false;
        if (args.length == 0) {
            System.exit(Mean.runSelfTest());
        } else if (args.length == 2) {
            try {
                final int n1 = Integer.valueOf(args[0]);
                final int n2 = Integer.valueOf(args[1]);
                System.out.println(Mean.mean(n1, n2));
            } catch (final NumberFormatException e) {
                fail = true;
                System.err.printf("error: Not an integer: %s%n",
                                  e.getMessage());
            }
        } else {
            fail = true;
            System.err.println("error: Wrong number of arguments");
        }
        if (fail) {
            System.err.printf("usage: java %s [N1 N2]%n",
                              Mean.class.getCanonicalName());
            System.err.println("Computes the mean of the integers N1 and N2.");
            System.err.println("If called without arguments, run a self-test.");
            System.exit(1);
        }
    }

    /**
     * Computes the mean of two integers.
     *
     * The result is rounded towards zero.
     *
     * @param n1
     *         the first number
     *
     * @param n2
     *         the second number
     *
     * @returns
     *         floor((n1 + n2) / 2)
     *
     */
    private static int mean(final int n1, final int n2) {
        return (n1 & n2) + (n1 ^ n2) / 2;
    }

    /**
     * Runs a self-test over a few corner-cases.
     *
     * A message for each test is printed to the standard output stream.
     *
     * @returns
     *         number of failed tests
     *
     */
    private static int runSelfTest() {
        final int[][] testCases = {
            // tuples (n1, n1, expected)
            { 0,  0,  0},
            { 1,  1,  1},
            { 1, -1,  0},
            {-1,  1,  0},
            { 1,  2,  1},
            { 1,  3,  2},
            { 1, -2,  0},
            { 1, -3, -1},
            {Integer.MAX_VALUE,  Integer.MAX_VALUE, Integer.MAX_VALUE},
            {Integer.MIN_VALUE,  Integer.MIN_VALUE, Integer.MIN_VALUE},
            {Integer.MAX_VALUE, -Integer.MAX_VALUE,                 0},
        };
        int failures = 0;
        System.out.printf("%12s%12s%12s%12s%12s%n",
                          "n1", "n2", "expected", "actual", "result");
        System.out.println();
        for (final int[] testCase : testCases) {
            final int n1 = testCase[0];
            final int n2 = testCase[1];
            final int expected = testCase[2];
            final int actual = Mean.mean(n1, n2);
            System.out.printf("%12d%12d%12d%12d%12s%n",
                              n1, n2, expected, actual,
                              (expected == actual) ? "passed" : "failed");
            if (expected != actual) { failures++; }
        }
        return failures;
    }
}
